@@ -72,15 +72,31 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 posterior0 <- 1/(1 + exp(score))
                 posterior1 <- 1 - posterior0
                 ric("other", xtrain, ytrain, Si, p0 = p0, p1 = p1, posterior0 = posterior0, posterior1 = posterior1, deg = function(i) {
-                    i
+                  i
                 })
             })
         }
 
+
+        if (criterion == "aic") {
+            subspace.list <- sapply(1:B2, function(i) {
+                Si <- S[, i][!is.na(S[, i])]  # current subspace
+                calc_aic(xtrain, ytrain, Si)
+            })
+        }
+
         if (criterion == "ebic") {
             subspace.list <- sapply(1:B2, function(i) {
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
-                calc_BIC(xtrain, ytrain, Si, D = 0, K = 0, debug = F, gam = gam)
+                # calc_BIC(xtrain, ytrain, Si, D = 0, K = 0, debug = F, gam = gam)
+                calc_ebic(xtrain, ytrain, Si, gam)
+            })
+        }
+
+        if (criterion == "bic") {
+            subspace.list <- sapply(1:B2, function(i) {
+                Si <- S[, i][!is.na(S[, i])]  # current subspace
+                calc_ebic(xtrain, ytrain, Si, gam = 0)
             })
         }
 
@@ -90,7 +106,7 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 xtrain.r <- xtrain[, Si, drop = F]
                 mean(as.numeric(I(predict(glm(y ~ ., data = data.frame(x = xtrain.r, y = ytrain), family = "binomial"), data.frame(x = xtrain.r)) >
-                                      0)) != ytrain, na.rm = TRUE)
+                  0)) != ytrain, na.rm = TRUE)
             })
         }
 
@@ -101,7 +117,7 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 xtrain.r <- xtrain[, Si, drop = F]
                 xval.r <- xval[, Si, drop = F]
                 mean(as.numeric(I(predict(glm(y ~ ., data = data.frame(x = xtrain.r, y = ytrain), family = "binomial"), data.frame(x = xval.r)) >
-                                      0)) != yval, na.rm = TRUE)
+                  0)) != yval, na.rm = TRUE)
             })
         }
 
@@ -111,8 +127,8 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 # the last row is training error for each i in 1:B2
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 mean(sapply(1:cv, function(j) {
-                    fit <- glm(y ~ ., data = data.frame(x = xtrain[-folds[[j]], Si, drop = F], y = ytrain[-folds[[j]]]), family = "binomial")
-                    mean(as.numeric(I(predict(fit, data.frame(x = xtrain[folds[[j]], Si, drop = F]))) > 0) != ytrain[folds[[j]]], na.rm = TRUE)
+                  fit <- glm(y ~ ., data = data.frame(x = xtrain[-folds[[j]], Si, drop = F], y = ytrain[-folds[[j]]]), family = "binomial")
+                  mean(as.numeric(I(predict(fit, data.frame(x = xtrain[folds[[j]], Si, drop = F]))) > 0) != ytrain[folds[[j]]], na.rm = TRUE)
                 }))
             })
         }
@@ -130,14 +146,6 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
     }
 
     if (base == "svm") {
-        if (criterion == "nric") {
-            subspace.list <- sapply(1:B2, function(i) {
-                # the last row is training error for each i in 1:B2
-                Si <- S[, i][!is.na(S[, i])]  # current subspace
-                -2*(p0*KL.divergence(xtrain[ytrain == 0, Si, drop = F], xtrain[ytrain == 1, Si, drop = F], k = kl.k)[kl.k] + p1*KL.divergence(xtrain[ytrain == 1, Si, drop = F], xtrain[ytrain == 0, Si, drop = F], k = kl.k)[kl.k]) +  + length(Si)*log(log(n))/sqrt(n)
-            })
-        }
-
         if (!is.character(kernel)) {
             kernel <- "linear"
         }
@@ -148,7 +156,7 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 xtrain.r <- xtrain[, Si, drop = F]
                 mean(as.numeric(predict(svm(x = xtrain.r, y = ytrain, kernel = kernel, type = "C-classification"), xtrain.r)) - 1 !=
-                         ytrain, na.rm = TRUE)
+                  ytrain, na.rm = TRUE)
             })
         }
 
@@ -159,7 +167,7 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 xtrain.r <- xtrain[, Si, drop = F]
                 xval.r <- xval[, Si, drop = F]
                 mean(as.numeric(predict(svm(x = xtrain.r, y = ytrain, kernel = kernel, type = "C-classification"), xval.r)) - 1 !=
-                         yval, na.rm = TRUE)
+                  yval, na.rm = TRUE)
             })
         }
 
@@ -169,8 +177,8 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 # the last row is training error for each i in 1:B2
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 mean(sapply(1:cv, function(j) {
-                    fit <- svm(x = xtrain[-folds[[j]], Si, drop = F], y = ytrain[-folds[[j]]], kernel = kernel, type = "C-classification")
-                    mean(as.numeric(predict(fit, xtrain[folds[[j]], Si, drop = F])) - 1 != ytrain[folds[[j]]], na.rm = TRUE)
+                  fit <- svm(x = xtrain[-folds[[j]], Si, drop = F], y = ytrain[-folds[[j]]], kernel = kernel, type = "C-classification")
+                  mean(as.numeric(predict(fit, xtrain[folds[[j]], Si, drop = F])) - 1 != ytrain[folds[[j]]], na.rm = TRUE)
                 }))
             })
         }
@@ -222,8 +230,8 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 # the last row is training error for each i in 1:B2
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 mean(sapply(1:cv, function(j) {
-                    fit <- randomForest(x = xtrain[-folds[[j]], Si, drop = F], y = factor(ytrain[-folds[[j]]]))
-                    mean((as.numeric(predict(fit, xtrain[folds[[j]], Si, drop = F])) - 1) != ytrain[folds[[j]]], na.rm = TRUE)
+                  fit <- randomForest(x = xtrain[-folds[[j]], Si, drop = F], y = factor(ytrain[-folds[[j]]]))
+                  mean((as.numeric(predict(fit, xtrain[folds[[j]], Si, drop = F])) - 1) != ytrain[folds[[j]]], na.rm = TRUE)
                 }))
             })
         }
@@ -255,7 +263,7 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 Si <- matrix(S[, i][!is.na(S[, i])], nrow = d)  # current subspace
                 xtrain.r <- xtrain[, Si, drop = F]
                 knn.test <- sapply(k, function(j) {
-                    mean(knn.cv(xtrain.r, ytrain, j, use.all = FALSE) != ytrain, na.rm = TRUE)
+                  mean(knn.cv(xtrain.r, ytrain, j, use.all = FALSE) != ytrain, na.rm = TRUE)
                 })
                 min(knn.test)
             })
@@ -281,8 +289,8 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 xval.r <- xval[, Si, drop = F]
 
                 knn.test <- sapply(k, function(j) {
-                    fit <- knn3(x = xtrain.r, y = factor(ytrain), k = j, use.all = FALSE)
-                    mean(predict(fit, xval.r, type = "class") != yval)
+                  fit <- knn3(x = xtrain.r, y = factor(ytrain), k = j, use.all = FALSE)
+                  mean(predict(fit, xval.r, type = "class") != yval)
                 })
                 min(knn.test)
             })
@@ -301,6 +309,32 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
             ytrain.pred <- predict(fit, xtrain.r, type = "class")
         }
 
+        if (criterion == "cv") {
+            folds <- createFolds(ytrain, k = cv)
+            subspace.list <- sapply(1:B2, function(i) {
+                # the last row is training error for each i in 1:B2
+                Si <- S[, i][!is.na(S[, i])]  # current subspace
+                knn.test <- sapply(k, function(l) {
+                    mean(sapply(1:cv, function(j) {
+                        mean(predict(knn3(x = xtrain[-folds[[j]], Si, drop = F], y = factor(ytrain[-folds[[j]]]), k = l, use.all = FALSE), xtrain[folds[[j]], Si, drop = F], type = "class") != ytrain[folds[[j]]], na.rm = TRUE)
+                    }))
+                })
+                min(knn.test)
+            })
+
+            i0 <- which.min(subspace.list)
+            S <- S[!is.na(S[, i0]), i0]  # final optimal subspace
+
+            xtrain.r <- xtrain[, S, drop = F]
+
+            knn.test <- sapply(k, function(j) {
+                mean(knn.cv(xtrain.r, ytrain, j, use.all = FALSE) != ytrain, na.rm = TRUE)
+            })
+            k.op <- k[which.min(knn.test)]
+            fit <- knn3(x = xtrain.r, y = factor(ytrain), k = k.op, use.all = FALSE)
+            ytrain.pred <- predict(fit, xtrain.r, type = "class")
+        }
+
         if (criterion == "training") {
             stop("minimizing training error is not available when base = \"knn\", please choose other criteria")
         }
@@ -344,9 +378,9 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 # the last row is training error for each i in 1:B2
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 mean(sapply(1:cv, function(j) {
-                    fit <- rpart(y ~ ., data = data.frame(x = xtrain[-folds[[j]], Si, drop = F], y = ytrain[-folds[[j]]]), method = "class")
-                    mean((as.numeric(predict(fit, data = data.frame(x = xtrain[folds[[j]], Si, drop = F]), type = "class")) - 1) != ytrain[folds[[j]]],
-                         na.rm = TRUE)
+                  fit <- rpart(y ~ ., data = data.frame(x = xtrain[-folds[[j]], Si, drop = F], y = ytrain[-folds[[j]]]), method = "class")
+                  mean((as.numeric(predict(fit, data = data.frame(x = xtrain[folds[[j]], Si, drop = F]), type = "class")) - 1) != ytrain[folds[[j]]],
+                    na.rm = TRUE)
                 }))
             })
         }
@@ -400,15 +434,28 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
 
         if (criterion == "ebic") {
             subspace.list <- sapply(1:B2, function(i) {
-                # the last row is training error for each i in 1:B2
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
-                calc_BIC(xtrain, ytrain, Si, D = 0, K = 0, debug = F, gam = gam)
+                # calc_BIC(xtrain, ytrain, Si, D = 0, K = 0, debug = F, gam = gam)
+                calc_ebic(xtrain, ytrain, Si, gam)
+            })
+        }
+
+        if (criterion == "bic") {
+            subspace.list <- sapply(1:B2, function(i) {
+                Si <- S[, i][!is.na(S[, i])]  # current subspace
+                calc_ebic(xtrain, ytrain, Si, gam = 0)
+            })
+        }
+
+        if (criterion == "aic") {
+            subspace.list <- sapply(1:B2, function(i) {
+                Si <- S[, i][!is.na(S[, i])]  # current subspace
+                calc_aic(xtrain, ytrain, Si)
             })
         }
 
         if (criterion == "training") {
             subspace.list <- sapply(1:B2, function(i) {
-                # the last row is training error for each i in 1:B2
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 xtrain.r <- xtrain[, Si, drop = F]
                 mean(predict(lda(x = xtrain.r, grouping = ytrain), xtrain.r)$class != ytrain, na.rm = TRUE)
@@ -421,8 +468,8 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 # the last row is training error for each i in 1:B2
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 mean(sapply(1:cv, function(j) {
-                    mean(predict(lda(x = xtrain[-folds[[j]], Si, drop = F], grouping = ytrain[-folds[[j]]]), xtrain[folds[[j]], Si, drop = F])$class !=
-                             ytrain[folds[[j]]], na.rm = TRUE)
+                  mean(predict(lda(x = xtrain[-folds[[j]], Si, drop = F], grouping = ytrain[-folds[[j]]]), xtrain[folds[[j]], Si, drop = F])$class !=
+                    ytrain[folds[[j]]], na.rm = TRUE)
                 }))
             })
         }
@@ -454,7 +501,7 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
                 # print(i) the last row is training error for each i in 1:B2
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 ric("qda", xtrain, ytrain, Si, mu0.mle = mu0.mle, mu1.mle = mu1.mle, Sigma0.mle = Sigma0.mle, Sigma1.mle = Sigma1.mle,
-                    p0 = p0, p1 = p1)
+                  p0 = p0, p1 = p1)
             })
         }
 
@@ -468,13 +515,6 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
             })
         }
 
-        if (criterion == "ebic") {
-            subspace.list <- sapply(1:B2, function(i) {
-                # the last row is training error for each i in 1:B2
-                Si <- S[, i][!is.na(S[, i])]  # current subspace
-                calc_BIC(xtrain, ytrain, Si, D = 0, K = 0, debug = F, gam = gam)
-            })
-        }
 
         if (criterion == "training") {
             subspace.list <- sapply(1:B2, function(i) {
@@ -489,8 +529,8 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
             subspace.list <- sapply(1:B2, function(i) {
                 Si <- S[, i][!is.na(S[, i])]  # current subspace
                 mean(sapply(1:cv, function(j) {
-                    mean(predict(qda(x = xtrain[-folds[[j]], Si, drop = F], grouping = ytrain[-folds[[j]]]), xtrain[folds[[j]], Si, drop = F])$class !=
-                             ytrain[folds[[j]]], na.rm = TRUE)
+                  mean(predict(qda(x = xtrain[-folds[[j]], Si, drop = F], grouping = ytrain[-folds[[j]]]), xtrain[folds[[j]], Si, drop = F])$class !=
+                    ytrain[folds[[j]]], na.rm = TRUE)
                 }))
             })
         }
@@ -508,4 +548,5 @@ RaSubset <- function(xtrain, ytrain, xval, yval, B2, S, base, k, criterion, cv,
     }
 
     return(list(fit = fit, ytrain.pred = as.numeric(ytrain.pred) - 1, subset = S))
+
 }
