@@ -462,5 +462,26 @@ RaSubsetsc_c <- function(xtrain, ytrain, xval, yval, B2, S, model, k, criterion,
     S <- S[!is.na(S[, i0]), i0]  # final optimal subspace
   }
 
+  if (model == "kernelknn") {
+    if (criterion == "cv") {
+      folds <- createFolds(ytrain, cv)
+      ytrain <- ytrain+1
+      subspace.list <- sapply(1:B2, function(i) {
+        d <- length(S[, i][!is.na(S[, i])])  # subspace size
+        Si <- matrix(S[, i][!is.na(S[, i])], nrow = d)  # current subspace
+        xtrain.r <- xtrain[, Si, drop = F]
+        knn.test <- sapply(1:cv, function(j) {
+          ypred <- KernelKnn(data = xtrain.r[-folds[[j]], , drop = F], TEST_data = xtrain.r[folds[[j]], , drop = F], y = ytrain[-folds[[j]]], k = k, regression = F, Levels = 1:max(ytrain), ...)
+          mean(apply(ypred, 1, function(x){which.max(x)}) != ytrain[folds[[j]]])
+        })
+
+        mean(knn.test)
+      })
+
+      i0 <- which.min(subspace.list)
+      S <- S[!is.na(S[, i0]), i0]  # final optimal subspace
+    }
+  }
+
     return(list(subset = S))
 }
